/************************************************************************
 *   IRC - Internet Relay Chat, m_core/m_topic.c
 *   This file is copyright (C) 2001 Andrew Suffield
 *                                    <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "ircd.h"
#include "common.h"
#include "numeric.h"
#include "irc_string.h"
#include "send.h"
#include "umodes.h"
#include "msg.h"
#include "handlers.h"
#include "channel.h"
#include "hash.h"
#include "struct.h"
#include "memory.h"
#include "list.h"
#include "extra.h"

#include <string.h>
#include <stdlib.h>

int m_invite(struct Client *, struct Client *, int, char **);

struct Message invite_msgtab = {
  "INVITE", 0, 3, 0, MFLG_SLOW, 0,
  {m_unregistered, m_invite, m_invite},
  NULL, NULL
};

/*
** m_invite
**      parv[0] - sender prefix
**      parv[1] - user to invite
**      parv[2] - channel number
*/
int
m_invite(struct Client *cptr, struct Client *sptr, int parc, char *parv[])
{
  struct Client *acptr;
  struct Channel *chptr;
  int need_invite = 0;
  time_t until = 0;

  if (parc < 3 || *parv[1] == '\0')
    {
      sendto_one(sptr, form_str(ERR_NEEDMOREPARAMS),
		 me.name, parv[0], "INVITE");
      return -1;
    }

#ifdef SERVERHIDE
#ifdef LOCAL_CHANNELS
  if (!IsSetOperGod(sptr) && parv[2][0] == '&')
    {
      sendto_one(sptr,
		 ":%s NOTICE %s :INVITE to local channels is disabled in this server",
		 me.name, parv[0]);
      return 0;
    }
#endif
#endif

  /* The length of time they may stay on the channel */
  if (parc > 3)
    until = atoi(parv[3]);

  /* A little sanity test here */
  if (!sptr->user)
    return 0;

  if (!(acptr = find_person(parv[1], (struct Client *) NULL)))
    {
      sendto_one(sptr, form_str(ERR_NOSUCHNICK), me.name, parv[0], parv[1]);
      return 0;
    }

  if (!check_channel_name(parv[2]))
    {
      sendto_one(sptr, form_str(ERR_BADCHANNAME),
		 me.name, parv[0], (unsigned char *) parv[2]);
      return 0;
    }

  if (!IsChannelName(parv[2]))
    {
      if (MyClient(sptr))
	sendto_one(sptr, form_str(ERR_NOSUCHCHANNEL),
		   me.name, parv[0], parv[2]);
      return 0;
    }

  if (!(chptr = hash_find_channel(parv[2], NullChn)))
    {
      if (MyClient(sptr))
	sendto_one(sptr, form_str(ERR_NOSUCHCHANNEL),
		   me.name, parv[0], parv[2]);
      return 0;
    }

  /* By this point, chptr is non NULL */

  if (!IsMember(sptr, chptr) && !HasUmode(sptr, UMODE_GOD))
    {
      if (MyClient(sptr))
	sendto_one(sptr, form_str(ERR_NOTONCHANNEL),
		   me.name, parv[0], parv[2]);
      return 0;
    }

  if (chptr && (chptr->mode_flags & CMODE_INVITEONLY))
    {
      need_invite = 1;

      if (!is_chan_op(sptr, chptr) && !HasUmode(sptr, UMODE_GOD))
	{
	  if (MyClient(sptr))
	    sendto_one(sptr, form_str(ERR_CHANOPRIVSNEEDED),
		       me.name, parv[0], parv[2]);
	  return -1;
	}
    }

  if (IsMember(acptr, chptr))
    {
      /* The target is already on the channel. Update their time to stay if
       *  they are only here on a temp invite
       *  -- asuffield
       */

      Link *lp;

      if (until)
	for (lp = chptr->tmemberlist; lp; lp = lp->next)
	  if (lp->value.tiptr->cptr == acptr)
	    {
	      lp->value.tiptr->until = CurrentTime + until;
	      sendto_one(sptr, form_str(RPL_RESETINVITE),
			 me.name, parv[0],
			 acptr->name, chptr->chname, lp->value.tiptr->until);
	      sendto_one(acptr, form_str(RPL_INVITEEXPIRE),
			 me.name, acptr->name,
			 chptr->chname, lp->value.tiptr->until);
	      return 0;
	    }

      if (MyClient(sptr))
	sendto_one(sptr, form_str(ERR_USERONCHANNEL),
		   me.name, parv[0], parv[1], parv[2]);
      return 0;
    }

  if (HasUmode(sptr, UMODE_NOINVITE))
    {
      if (MyClient(sptr))
	sendto_one(sptr, form_str(ERR_SOURCENINVITE), me.name, parv[0]);
      return 0;

    }

  if (HasUmode(acptr, UMODE_NOINVITE))
    {
      if (MyClient(sptr))
	sendto_one(sptr, form_str(ERR_TARGETNINVITE), me.name, parv[0]);
      return 0;

    }

  /*
   * due to some whining I've taken out the need for the channel
   * being +i before sending an INVITE. It was intentionally done this
   * way, it makes no sense (to me at least) letting the server send
   * an unnecessary invite when a channel isn't +i !
   * bah. I can't be bothered arguing it
   * -Dianora
   */
  if (MyConnect(sptr) /* && need_invite */ )
    {
      sendto_one(sptr, form_str(RPL_INVITING), me.name, parv[0],
		 acptr->name, ((chptr) ? (chptr->chname) : parv[2]));
      if (acptr->user->away)
	sendto_one(sptr, form_str(RPL_AWAY), me.name, parv[0],
		   acptr->name, acptr->user->away);
    }

  if (MyConnect(acptr) && need_invite)
    add_invite(acptr, chptr, until);

  sendto_prefix_one(acptr, sptr, ":%s INVITE %s %s %lu",
		    parv[0], acptr->name, parv[2], until);
  return 0;
}

time_t
_check_temp_invites(time_t currenttime)
{
  time_t next_due = -1;
  struct Channel *chptr;

  for (chptr = channel; chptr; chptr = chptr->nextch)
    {
      register Link **lp, *tmp;
      for (lp = &chptr->tmemberlist; (tmp = *lp); lp = &tmp->next)
	if (currenttime >= tmp->value.tiptr->until)
	  {
	    /* Your time just expired */
	    struct Client *acptr = tmp->value.tiptr->cptr;

	    /* Got a lifeline? */
	    if (!(HasUmode(acptr, UMODE_IMMUNE) || is_invex(acptr, chptr)))
	      {
		/* Nope */
		char *aparv[] = { NULL, NULL, NULL, "Invitation expired" };
		aparv[0] = me.name;
		aparv[1] = chptr->chname;
		aparv[2] = acptr->name;
/* 		m_kick(&me, &me, 4, aparv); */
	      }
	    else
	      {
		/* Remove the entry manually if they're safe (also done by m_kick()) */
		*lp = tmp->next;
		MyFree(tmp->value.tiptr);
		free_link(tmp);
		break;
	      }
	  }
	else if ((tmp->value.tiptr->until < next_due) || (next_due == -1))
	  next_due = tmp->value.tiptr->until;
    }

  return next_due;
}
